{% extends 'w3bsite/classes/apps/defaults/html/base.html' %}
{% load static %}

<!-- LEFTBAR -->
{% block leftbar %}

{% endblock %}

<!-- RIGHT BAR -->
{% block rightbar %}

{% endblock %}

<!-- CONTENT -->
{% block content %}

{% endblock %}